@@ -27,6 +27,8 @@
 #include <dix-config.h>
 #endif
 
+#include <map>
+
 #include <rfb/SDesktop.h>
 #include <rfb/HTTPServer.h>
 #include <rfb/PixelBuffer.h>
@@ -39,6 +41,9 @@ extern "C" {
 #define class c_class
 #include <scrnintstr.h>
 #include <os.h>
+#ifdef RANDR
+#include <randrstr.h>
+#endif
 #undef class
 }
 
@@ -49,7 +54,7 @@ namespace rfb {
 namespace network { class TcpListener; class Socket; }
 
 class XserverDesktop : public rfb::SDesktop, public rfb::FullFramePixelBuffer,
-                       public rfb::ColourMap, public rdr::Substitutor,
+                       public rdr::Substitutor,
                        public rfb::VNCServerST::QueryConnectionHandler {
 public:
 
@@ -60,12 +65,10 @@ public:
   virtual ~XserverDesktop();
 
   // methods called from X server code
-  void serverReset(ScreenPtr pScreen);
   void blockUpdates();
   void unblockUpdates();
   void setFramebuffer(int w, int h, void* fbptr, int stride);
-  void setColormap(ColormapPtr cmap);
-  void setColourMapEntries(ColormapPtr pColormap, int ndef, xColorItem* pdef);
+  void refreshScreenLayout();
   void bell();
   void serverCutText(const char* str, int len);
   void setDesktopName(const char* name);
@@ -101,17 +104,11 @@ public:
   virtual void keyEvent(rdr::U32 key, bool down);
   virtual void clientCutText(const char* str, int len);
   virtual rfb::Point getFbSize() { return rfb::Point(width(), height()); }
-#ifdef RANDR
   virtual unsigned int setScreenLayout(int fb_width, int fb_height,
                                        const rfb::ScreenSet& layout);
-#endif
 
   // rfb::PixelBuffer callbacks
   virtual void grabRegion(const rfb::Region& r);
-  virtual int getStride() const;
-
-  // rfb::ColourMap callbacks
-  virtual void lookup(int index, int* r, int* g, int* b);
 
   // rdr::Substitutor callback
   virtual char* substitute(const char* varName);
@@ -122,15 +119,16 @@ public:
                                                         char** reason);
 
 private:
-  void setColourMapEntries(int firstColour, int nColours);
+  rfb::ScreenSet computeScreenLayout();
+#ifdef RANDR
+  RRModePtr findRandRMode(RROutputPtr output, int width, int height);
+#endif
+
   ScreenPtr pScreen;
-  InputDevice *inputDevice;
   rfb::VNCServerST* server;
   rfb::HTTPServer* httpServer;
   network::TcpListener* listener;
   network::TcpListener* httpListener;
-  ColormapPtr cmap;
-  int stride_;
   bool deferredUpdateTimerSet;
   bool grabbing;
   bool ignoreHooks_;
@@ -139,5 +137,12 @@ private:
   void* queryConnectId;
   rfb::CharArray queryConnectAddress;
   rfb::CharArray queryConnectUsername;
+
+#ifdef RANDR
+  typedef std::map<RROutputPtr, rdr::U32> OutputIdMap;
+  OutputIdMap outputIdMap;
+#endif
+
+  rfb::Point oldCursorPos;
 };
 #endif
